/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes;

import clenotes.Command;
import clenotes.Logger;
import clenotes.Option;
import com.google.common.base.Objects;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Conversions;

/**
 * Command line syntax is
 * 
 * [global options] [command [command-options] ...]
 * 
 * option is --optionaname
 * or --optionname=value1,value2
 * #no spaces between values,
 */
@SuppressWarnings("all")
public class CommandLineArguments {
  private static List<String> args;
  
  public static Map<String,Option> globalOptionMap = CollectionLiterals.<String, Option>newLinkedHashMap();
  
  public static LinkedHashMap<Object,Object> commandMap = CollectionLiterals.<Object, Object>newLinkedHashMap();
  
  public static void parse(final String[] _args) {
    CommandLineArguments.args = ((List<String>)Conversions.doWrapArray(_args));
    int i = 0;
    boolean commandInProgress = false;
    String commandNameInProgress = "";
    int _size = CommandLineArguments.args.size();
    boolean _lessThan = (i < _size);
    boolean _while = _lessThan;
    while (_while) {
      {
        String arg = CommandLineArguments.args.get(i);
        Logger.log(("Argument: " + arg));
        boolean _startsWith = arg.startsWith("--");
        boolean _not = (!_startsWith);
        if (_not) {
          commandInProgress = true;
          Command cmd = new Command();
          cmd.setName(arg);
          commandNameInProgress = arg;
          CommandLineArguments.commandMap.put(arg, cmd);
        } else {
          final Option option = CommandLineArguments.getNextOption(arg);
          if (commandInProgress) {
            Object _get = CommandLineArguments.commandMap.get(commandNameInProgress);
            Command cmd_1 = ((Command) _get);
            Map<String,Option> opts = cmd_1.getOptionMap();
            String _name = option.getName();
            opts.put(_name, option);
          } else {
            String _name_1 = option.getName();
            CommandLineArguments.globalOptionMap.put(_name_1, option);
          }
        }
        i = (i + 1);
      }
      int _size_1 = CommandLineArguments.args.size();
      boolean _lessThan_1 = (i < _size_1);
      _while = _lessThan_1;
    }
    Logger.log(("Global Options in command line: " + CommandLineArguments.globalOptionMap));
    Logger.log(("Commands  in command line: " + CommandLineArguments.commandMap));
  }
  
  private static Option getNextOption(final String _argument) {
    String argument = _argument.replaceFirst("--", "");
    Option option = null;
    boolean _contains = argument.contains("=");
    if (_contains) {
      int _indexOf = argument.indexOf("=");
      String argumentName = argument.substring(0, _indexOf);
      int _indexOf_1 = argument.indexOf("=");
      int _plus = (_indexOf_1 + 1);
      String argumentValues = argument.substring(_plus);
      List<String> values = new Vector<String>();
      boolean _and = false;
      boolean _notEquals = (!Objects.equal(argumentName, "delim"));
      if (!_notEquals) {
        _and = false;
      } else {
        boolean _contains_1 = argumentValues.contains(",");
        _and = _contains_1;
      }
      if (_and) {
        List<String> _values = (List<String>)Conversions.doWrapArray(argumentValues.split(","));
        for (final String v : _values) {
          values.add(v);
        }
      } else {
        values.add(argumentValues);
      }
      boolean _containsKey = CommandLineArguments.globalOptionMap.containsKey(argumentName);
      if (_containsKey) {
        Option _get = CommandLineArguments.globalOptionMap.get(argumentName);
        option = ((Option) _get);
        List<String> existingValues = option.getValues();
        existingValues.addAll(values);
      } else {
        Option _option = new Option();
        option = _option;
        option.setName(argumentName);
        option.setValues(values);
      }
    } else {
      Option _option_1 = new Option();
      option = _option_1;
      option.setName(argument);
    }
    int _size = CommandLineArguments.args.size();
    CommandLineArguments.args.subList(1, _size);
    return option;
  }
  
  public static boolean hasOption(final String optionName) {
    boolean _xblockexpression = false;
    {
      Option option = CommandLineArguments.globalOptionMap.get(optionName);
      _xblockexpression = (!Objects.equal(option, null));
    }
    return _xblockexpression;
  }
  
  public static String getValue(final String optionName) {
    String _xblockexpression = null;
    {
      Option _get = CommandLineArguments.globalOptionMap.get(optionName);
      Option option = ((Option) _get);
      String value = null;
      boolean _notEquals = (!Objects.equal(option, null));
      if (_notEquals) {
        String _value = option.getValue();
        value = _value;
      }
      _xblockexpression = value;
    }
    return _xblockexpression;
  }
  
  public static List<String> getValues(final String optionName) {
    List<String> _xblockexpression = null;
    {
      Option _get = CommandLineArguments.globalOptionMap.get(optionName);
      Option option = ((Option) _get);
      List<String> values = null;
      boolean _notEquals = (!Objects.equal(option, null));
      if (_notEquals) {
        List<String> _values = option.getValues();
        values = _values;
      }
      _xblockexpression = values;
    }
    return _xblockexpression;
  }
}
