/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes;

import clenotes.CLENotes;
import clenotes.CommandLineArguments;
import clenotes.Configuration;
import clenotes.Logger;
import clenotes.TestNotesAvailability;
import clenotes.commands.Help;
import clenotes.utils.ExtensionMethods;
import clenotes.utils.Input;
import com.google.common.base.Objects;
import java.io.Console;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import lotus.domino.NotesThread;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.StringExtensions;

@SuppressWarnings("all")
public class Main {
  public static String printHeader() {
    String _xifexpression = null;
    boolean _containsKey = CommandLineArguments.globalOptionMap.containsKey("noheader");
    boolean _not = (!_containsKey);
    if (_not) {
      _xifexpression = InputOutput.<String>println(Configuration.HEADER);
    }
    return _xifexpression;
  }
  
  public static void main(final String[] args) {
    try {
      int returnValue = 0;
      long startTime = System.currentTimeMillis();
      try {
        for (final String arg : args) {
          boolean _equals = Objects.equal(arg, "--log");
          if (_equals) {
            Configuration.setLogEnabled(true);
          }
        }
        try {
          new TestNotesAvailability();
        } catch (final Throwable _t) {
          if (_t instanceof NoClassDefFoundError) {
            final NoClassDefFoundError exc = (NoClassDefFoundError)_t;
            InputOutput.<String>println("Notes classes not found.");
            final String answer = Input.prompt("Install Notes classes (yes/no) [yes]? ");
            boolean _or = false;
            boolean _isEmpty = answer.isEmpty();
            if (_isEmpty) {
              _or = true;
            } else {
              boolean _equals_1 = Objects.equal(answer, "yes");
              _or = _equals_1;
            }
            if (_or) {
              Main.setupNotesClasses();
            } else {
              InputOutput.<String>println("Notes classes not installed.");
            }
            return;
          } else {
            throw Exceptions.sneakyThrow(_t);
          }
        }
        Configuration.load();
        final List<String> argsList = (List<String>)Conversions.doWrapArray(args);
        String _join = IterableExtensions.join(argsList, " ");
        String _plus = ("CLENOTES START: " + _join);
        Logger.log(_plus);
        CommandLineArguments.parse(args);
        Configuration.parse();
        Configuration.printConfigFileEntries();
        Main.printHeader();
        LinkedHashMap<Object,Object> commandMap = CommandLineArguments.commandMap;
        boolean _and = false;
        boolean _isEmpty_1 = commandMap.isEmpty();
        if (!_isEmpty_1) {
          _and = false;
        } else {
          boolean _isEmpty_2 = CommandLineArguments.globalOptionMap.isEmpty();
          _and = _isEmpty_2;
        }
        if (_and) {
          InputOutput.<String>println("No commands specified. Use --help option for usage.");
          return;
        }
        boolean _or_1 = false;
        boolean _containsKey = CommandLineArguments.globalOptionMap.containsKey("help");
        if (_containsKey) {
          _or_1 = true;
        } else {
          boolean _containsKey_1 = commandMap.containsKey("help");
          _or_1 = _containsKey_1;
        }
        if (_or_1) {
          int _execute = Help.execute(null, null);
          returnValue = _execute;
          return;
        }
        Logger.log("Starting CLENotes thread..");
        CLENotes cleNotes = new CLENotes();
        String threadName = "CLENotesThreadRunnable";
        NotesThread notesThread = new NotesThread(cleNotes, threadName);
        notesThread.start();
        notesThread.join();
        int _returnCode = cleNotes.getReturnCode();
        returnValue = _returnCode;
        Logger.log("CLENotes thread ended..");
      } finally {
        long endTime = System.currentTimeMillis();
        String execTime = (("Execution time: " + Double.valueOf(((endTime - startTime) / 1000.0))) + " seconds.");
        boolean _hasOption = CommandLineArguments.hasOption("exec-time");
        if (_hasOption) {
          InputOutput.println();
          InputOutput.<String>println(execTime);
        }
        Logger.log(execTime);
        Logger.log("CLENOTES END");
        System.exit(returnValue);
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  private static void setupNotesClasses() {
    final String OSname = System.getProperty("os.name");
    Logger.log(("OS name: " + OSname));
    LinkedList<String> notesDirs = CollectionLiterals.<String>newLinkedList();
    final String libDir = "/jvm/lib/ext/Notes.jar";
    boolean _matched = false;
    if (!_matched) {
      String _lowerCase = OSname.toLowerCase();
      int _indexOf = _lowerCase.indexOf("win");
      boolean _greaterEqualsThan = (_indexOf >= 0);
      if (_greaterEqualsThan) {
        _matched=true;
        notesDirs.add(("C:/Program Files/IBM/Lotus/Notes" + libDir));
        notesDirs.add(("C:/Lotus/Notes" + libDir));
        notesDirs.add(("c:/notes" + libDir));
      }
    }
    if (!_matched) {
      String _lowerCase_1 = OSname.toLowerCase();
      int _indexOf_1 = _lowerCase_1.indexOf("linux");
      boolean _greaterEqualsThan_1 = (_indexOf_1 >= 0);
      if (_greaterEqualsThan_1) {
        _matched=true;
        notesDirs.add(("/opt/ibm/lotus/notes" + libDir));
      }
    }
    if (!_matched) {
      String _lowerCase_2 = OSname.toLowerCase();
      int _indexOf_2 = _lowerCase_2.indexOf("mac");
      boolean _greaterEqualsThan_2 = (_indexOf_2 >= 0);
      if (_greaterEqualsThan_2) {
        _matched=true;
        notesDirs.add(("/Applications/Notes.app" + libDir));
      }
    }
    if (!_matched) {
      {
        final String dir = Main.promptNotesJarLocation();
        boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(dir);
        if (_isNullOrEmpty) {
          return;
        }
        notesDirs.add(dir);
      }
    }
    boolean installed = Main.installNotesJar(notesDirs);
    boolean _while = (!installed);
    while (_while) {
      {
        final String dir = Main.promptNotesJarLocation();
        boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(dir);
        if (_isNullOrEmpty) {
          InputOutput.<String>println("Notes classes not installed.");
          return;
        }
        notesDirs.clear();
        notesDirs.add(dir);
        boolean _installNotesJar = Main.installNotesJar(notesDirs);
        installed = _installNotesJar;
      }
      _while = (!installed);
    }
  }
  
  private static String promptNotesJarLocation() {
    String _xblockexpression = null;
    {
      final String prompt = "What is the full path of Notes.jar? ";
      Console _console = System.console();
      String dir = _console.readLine(prompt);
      boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(dir);
      if (_isNullOrEmpty) {
        return null;
      }
      String dir2 = dir.toLowerCase();
      boolean _endsWith = dir2.endsWith("notes.jar");
      boolean _not = (!_endsWith);
      if (_not) {
        boolean _or = false;
        boolean _endsWith_1 = dir2.endsWith("/");
        boolean _not_1 = (!_endsWith_1);
        if (_not_1) {
          _or = true;
        } else {
          boolean _endsWith_2 = dir2.endsWith("\\");
          boolean _not_2 = (!_endsWith_2);
          _or = _not_2;
        }
        if (_or) {
          dir = (dir + "/");
        }
        dir = (dir + "Notes.jar");
      }
      _xblockexpression = dir;
    }
    return _xblockexpression;
  }
  
  private static boolean installNotesJar(final List<String> notesDirs) {
    InputOutput.<String>println("Locating Notes.jar... ");
    for (final String dir : notesDirs) {
      {
        final File file = new File(dir);
        InputOutput.<String>print((dir + ": "));
        boolean _exists = file.exists();
        if (_exists) {
          InputOutput.<String>println("found.");
          InputOutput.<String>print("Extracting classes... ");
          final int filesExtracted = Main.extractJar(file);
          String _plus = (Integer.valueOf(filesExtracted) + " classes extracted.");
          InputOutput.<String>println(_plus);
          InputOutput.<String>println("Notes classes installed.");
          return true;
        } else {
          InputOutput.<String>println("not found.");
        }
      }
    }
    return false;
  }
  
  private static int extractJar(final File notesJarFile) {
    try {
      int _xblockexpression = (int) 0;
      {
        byte[] buffer = new byte[1024];
        final File outputDir = new File("classes");
        FileInputStream _fileInputStream = new FileInputStream(notesJarFile);
        ZipInputStream jarFile = new ZipInputStream(_fileInputStream);
        ZipEntry entry = jarFile.getNextEntry();
        int fileCount = 0;
        int currentNameLength = 0;
        boolean _notEquals = (!Objects.equal(entry, null));
        boolean _while = _notEquals;
        while (_while) {
          {
            final String name = entry.getName();
            String _times = ExtensionMethods.times("\b \b", currentNameLength);
            String _plus = (_times + name);
            InputOutput.<String>print(_plus);
            int _length = name.length();
            currentNameLength = _length;
            File f = new File(outputDir, name);
            File _parentFile = f.getParentFile();
            _parentFile.mkdirs();
            FileOutputStream outputStream = new FileOutputStream(f);
            int size = 0;
            int _read = jarFile.read(buffer);
            int _size = size = _read;
            boolean _greaterThan = (_size > 0);
            boolean _while_1 = _greaterThan;
            while (_while_1) {
              {
                outputStream.write(buffer, 0, size);
                outputStream.flush();
              }
              int _read_1 = jarFile.read(buffer);
              int _size_1 = size = _read_1;
              boolean _greaterThan_1 = (_size_1 > 0);
              _while_1 = _greaterThan_1;
            }
            outputStream.close();
            fileCount = (fileCount + 1);
            ZipEntry _nextEntry = jarFile.getNextEntry();
            entry = _nextEntry;
          }
          boolean _notEquals_1 = (!Objects.equal(entry, null));
          _while = _notEquals_1;
        }
        InputOutput.println();
        _xblockexpression = fileCount;
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
