/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes.commands;

import clenotes.Command;
import clenotes.CommandLineArguments;
import clenotes.commands.ReadMail;
import clenotes.utils.DominoUtils;
import com.google.common.base.Objects;
import lotus.domino.Database;
import lotus.domino.DateTime;
import lotus.domino.Document;
import lotus.domino.DocumentCollection;
import lotus.domino.Session;
import lotus.domino.View;
import lotus.domino.ViewEntry;
import lotus.domino.ViewEntryCollection;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;

@SuppressWarnings("all")
public class List {
  public static Object execute(final Command cmd, final Session notesSession) {
    try {
      Object _xblockexpression = null;
      {
        String folder = cmd.getOptionValue("folder");
        Database mailDb = DominoUtils.getMailDatabase(notesSession);
        DocumentCollection docCollection = ((DocumentCollection) null);
        boolean _notEquals = (!Objects.equal(folder, null));
        if (_notEquals) {
          DocumentCollection _profileDocCollection = mailDb.getProfileDocCollection("PROFILE_THAT_SHOULD_NOT_EXIST");
          docCollection = _profileDocCollection;
          View folderView = mailDb.getView(folder);
          ViewEntryCollection viewEntryCollection = folderView.getAllEntries();
          ViewEntry viewEntry = viewEntryCollection.getFirstEntry();
          boolean _notEquals_1 = (!Objects.equal(viewEntry, null));
          boolean _while = _notEquals_1;
          while (_while) {
            {
              Document doc = viewEntry.getDocument();
              boolean _notEquals_2 = (!Objects.equal(doc, null));
              if (_notEquals_2) {
                docCollection.addDocument(doc);
              }
              ViewEntry _nextEntry = viewEntryCollection.getNextEntry();
              viewEntry = _nextEntry;
            }
            boolean _notEquals_2 = (!Objects.equal(viewEntry, null));
            _while = _notEquals_2;
          }
        } else {
          DateTime dt = null;
          String adjustDay = CommandLineArguments.getValue("adjust-day");
          boolean _notEquals_2 = (!Objects.equal(adjustDay, null));
          if (_notEquals_2) {
            DateTime _createDateTime = notesSession.createDateTime("Yesterday");
            dt = _createDateTime;
            int _parseInt = Integer.parseInt(adjustDay);
            int _minus = (-_parseInt);
            dt.adjustDay(_minus);
          }
          boolean _hasOption = cmd.hasOption("all");
          if (_hasOption) {
            DocumentCollection _search = mailDb.search("@All", dt);
            docCollection = _search;
          } else {
            String _userName = notesSession.getUserName();
            String _plus = ("From!=\"" + _userName);
            String _plus_1 = (_plus + "\" & (Form=\"Memo\" | Form=\"Reply\")");
            DocumentCollection _search_1 = mailDb.search(_plus_1, dt);
            docCollection = _search_1;
          }
        }
        Object _xifexpression = null;
        boolean _hasOption_1 = cmd.hasOption("checkMIME");
        if (_hasOption_1) {
          InputOutput.<String>println("Checking MIME emails...");
          int n = 1;
          Document doc = docCollection.getNthDocument(n);
          boolean _notEquals_3 = (!Objects.equal(doc, null));
          boolean _while_1 = _notEquals_3;
          while (_while_1) {
            {
              boolean hasNativeMIME = doc.hasItem("$NoteHasNativeMIME");
              if (hasNativeMIME) {
                Object deliveredDate = DominoUtils.getItemValue(doc, "DeliveredDate");
                String subject = doc.getItemValueString("Subject");
                String _plus_2 = (deliveredDate + ": ");
                String _plus_3 = (_plus_2 + subject);
                InputOutput.<String>println(_plus_3);
                String _attachmentNames = DominoUtils.getAttachmentNames(doc, notesSession, false);
                InputOutput.<String>println(_attachmentNames);
              }
              n = (n + 1);
              doc.recycle();
              Document _nthDocument = docCollection.getNthDocument(n);
              doc = _nthDocument;
            }
            boolean _notEquals_4 = (!Objects.equal(doc, null));
            _while_1 = _notEquals_4;
          }
        } else {
          _xifexpression = List.handleDocumentCollection(docCollection, mailDb, cmd, notesSession);
        }
        _xblockexpression = _xifexpression;
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static Object handleDocumentCollection(final DocumentCollection _docCollection, final Database mailDb, final Command cmd, final Session session) {
    try {
      Object _xblockexpression = null;
      {
        DocumentCollection docCollection = DominoUtils.sortDocumentCollectionIfNecessary(_docCollection, mailDb, cmd);
        String readIndex = cmd.getOptionValue("read");
        Object _xifexpression = null;
        boolean _equals = Objects.equal(readIndex, "*");
        if (_equals) {
          int n = 1;
          Document doc = _docCollection.getNthDocument(n);
          boolean _notEquals = (!Objects.equal(doc, null));
          boolean _while = _notEquals;
          while (_while) {
            {
              boolean isDeleted = ReadMail.readMail(doc, cmd, session);
              n = (n + 1);
              doc.recycle();
              Document _nthDocument = _docCollection.getNthDocument(n);
              doc = _nthDocument;
            }
            boolean _notEquals_1 = (!Objects.equal(doc, null));
            _while = _notEquals_1;
          }
        } else {
          Object _xblockexpression_1 = null;
          {
            int rIndex = (-1);
            boolean _notEquals_1 = (!Objects.equal(readIndex, null));
            if (_notEquals_1) {
              int _parseInt = Integer.parseInt(readIndex);
              rIndex = _parseInt;
            }
            java.util.List<?> mails = DominoUtils.getMails(docCollection, cmd, rIndex, false, session);
            Object numberOfMails = mails.get(0);
            Object _get = mails.get(1);
            String mailTxt = ((String) _get);
            Object _get_1 = mails.get(2);
            Document mailDoc = ((Document) _get_1);
            Object _xifexpression_1 = null;
            boolean _equals_1 = Objects.equal(mailDoc, null);
            if (_equals_1) {
              String _xblockexpression_2 = null;
              {
                String txt = "mails";
                boolean _equals_2 = Objects.equal(numberOfMails, Integer.valueOf(1));
                if (_equals_2) {
                  txt = "mail";
                }
                InputOutput.<String>println((((("Listing " + numberOfMails) + " ") + txt) + "."));
                _xblockexpression_2 = InputOutput.<String>println(mailTxt);
              }
              _xifexpression_1 = _xblockexpression_2;
            } else {
              _xifexpression_1 = Boolean.valueOf(ReadMail.readMail(mailDoc, cmd, session));
            }
            _xblockexpression_1 = _xifexpression_1;
          }
          _xifexpression = _xblockexpression_1;
        }
        _xblockexpression = _xifexpression;
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
