/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes.utils;

import clenotes.utils.ExtensionMethods;
import java.util.List;
import org.eclipse.xtext.xbase.lib.InputOutput;

@SuppressWarnings("all")
public class Output {
  private final static int defaultColumnWidth = 20;
  
  private final static int defaultScreenWidth = 80;
  
  private final static int defaultIndentation = 4;
  
  private final static int defaultLineStartIndentLevel = 1;
  
  /**
   * print messages in messagelist in single line per columns
   * format to be pretty
   */
  public static void prettyPrintln(final List<String> messageList) {
    Output.prettyPrintln(messageList, Output.defaultColumnWidth, Output.defaultScreenWidth, Output.defaultLineStartIndentLevel);
  }
  
  public static void prettyPrintln(final List<String> messageList, final int columnWidth, final int lineStartIndentLevel) {
    Output.prettyPrintln(messageList, columnWidth, Output.defaultScreenWidth, lineStartIndentLevel);
  }
  
  public static void prettyPrintln(final List<String> messageList, final int columnWidth, final int screenWidth, final int lineStartIndentLevel) {
    if ((lineStartIndentLevel != 0)) {
      String _times = ExtensionMethods.times(" ", (lineStartIndentLevel * Output.defaultIndentation));
      InputOutput.<String>print(_times);
    }
    int lineLength = 0;
    for (final String _msg : messageList) {
      {
        String msg = _msg;
        String _ljust = ExtensionMethods.ljust(msg, columnWidth);
        msg = _ljust;
        int _length = msg.length();
        int _plus = (lineLength + _length);
        lineLength = _plus;
        InputOutput.<String>print(msg);
      }
    }
    InputOutput.println();
  }
}
