/*
 * Copyright 2002, 2014 IBM Corp.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.  
 *
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
*/
package clenotes.commands

import clenotes.Command
import clenotes.CommandLineArguments
import clenotes.utils.DominoUtils
import lotus.domino.Database
import lotus.domino.DateTime
import lotus.domino.Document
import lotus.domino.DocumentCollection
import lotus.domino.Session

class List {

	def static execute(Command cmd, Session notesSession) {

		var folder = cmd.getOptionValue("folder")

		var mailDb = DominoUtils::getMailDatabase(notesSession)

		var docCollection = null as DocumentCollection
		if (folder != null) {

			//create empty doc collection using method below
			docCollection = mailDb.getProfileDocCollection("PROFILE_THAT_SHOULD_NOT_EXIST")

			//folder is View, use Views api
			//request from user:
			//1. Read each mail document under a given folder.
			//  For each document under the folder,
			//  2. Detach all its attachments
			//  3. Move the document to another given folder.
			var folderView = mailDb.getView(folder)
			var viewEntryCollection = folderView.getAllEntries()
			var viewEntry = viewEntryCollection.getFirstEntry()
			while (viewEntry != null) {
				var doc = viewEntry.getDocument()
				if (doc != null) {
					docCollection.addDocument(doc)

				}
				viewEntry = viewEntryCollection.getNextEntry()

			}

		} else {

			var DateTime dt = null

			var adjustDay = CommandLineArguments::getValue("adjust-day")
			if (adjustDay != null) {
				dt = notesSession.createDateTime("Yesterday")
				dt.adjustDay(-Integer::parseInt(adjustDay))
			}

			//If searching @All, all document returned, memos, reply, invitations etc
			if (cmd.hasOption("all")) {
				docCollection = mailDb.search("@All", dt)
			} else {
				docCollection = mailDb.search(
					"From!=\"" + notesSession.getUserName() + "\" & (Form=\"Memo\" | Form=\"Reply\")", dt)
			}

		}

		if (cmd.hasOption("checkMIME")) {
			println("Checking MIME emails...")
			var n = 1
			var doc = docCollection.getNthDocument(n)
			while (doc != null) {
				var hasNativeMIME = doc.hasItem("$NoteHasNativeMIME")
				if (hasNativeMIME) {
					var deliveredDate = DominoUtils::getItemValue(doc, "DeliveredDate")
					var subject = doc.getItemValueString("Subject")
					println(deliveredDate + ": " + subject)
					println(DominoUtils::getAttachmentNames(doc, notesSession, false))
				}
				n = n + 1
				doc.recycle()
				doc = docCollection.getNthDocument(n)
			}

		} else {
			handleDocumentCollection(docCollection, mailDb, cmd, notesSession)

		}

	}

	def static handleDocumentCollection(DocumentCollection _docCollection, Database mailDb, Command cmd, Session session) {

		//should sort be later? after DominoUtils::getMails?
		var docCollection = DominoUtils::sortDocumentCollectionIfNecessary(_docCollection, mailDb, cmd)
		var readIndex = cmd.getOptionValue("read")

		if (readIndex == "*") {
			var n = 1
			var doc = _docCollection.getNthDocument(n)
			while (doc != null) {
				var isDeleted = ReadMail::readMail(doc, cmd, session)
				n = n + 1
				doc.recycle()
				doc = _docCollection.getNthDocument(n)
			}

		} else {
			var rIndex = -1
			if (readIndex != null) {
				rIndex = Integer::parseInt(readIndex)
			}

			//var docCollection = DominoUtils::getMailDocCollection(_docCollection, mailDb, cmd)
			//docCollection = DominoUtils::sortDocumentCollectionIfNecessary(docCollection, mailDb, cmd)
			//return from getMails is a list		#[index, mailTxt, mailDocToBeRead]
			var mails = DominoUtils::getMails(docCollection, cmd, rIndex, false, session)
			var numberOfMails = mails.get(0)
			var mailTxt = mails.get(1) as String
			var mailDoc = mails.get(2) as Document

			if (mailDoc == null) {
				var txt = "mails"
				if(numberOfMails == 1) txt = "mail"
				println("Listing " + numberOfMails + " " + txt + ".")
				println(mailTxt)

			} else {
				ReadMail::readMail(mailDoc, cmd, session)

			}
		}
	}
}
